#!/usr/bin/perl -w -I.
use strict;

#  ----------------------------------------------------------------------
# | Open Archives Initiative Harvesting Framework version 1.1            |
# | Hussein Suleman                                                      |
# | September 2001                                                       |
#  ----------------------------------------------------------------------
# |  Virginia Polytechnic Institute and State University                 |
# |  Department of Computer Science                                      |
# |  Digital Library Research Laboratory                                 |
#  ----------------------------------------------------------------------

# clean XML by removing invalid characters, converting to
# Unicode, reparagraphing and escaping
# format XML from Perl structure
# check date formats

package Utility;

sub new
{
   my $classname = shift;
   my $self = { XMLindent => '   ' };

   my @upperentities = qw (nbsp iexcl cent pound curren yen brvbar sect 
                           uml copy ordf laquo not 173 reg macr deg plusmn 
                           sup2 sup3 acute micro para middot cedil supl 
                           ordm raquo frac14 half frac34 iquest Agrave 
                           Aacute Acirc Atilde Auml Aring AElig Ccedil 
                           Egrave Eacute Ecirc Euml Igrave Iacute Icirc 
                           Iuml ETH Ntilde Ograve Oacute Ocirc Otilde Ouml 
                           times Oslash Ugrave Uacute Ucirc Uuml Yacute 
                           THORN szlig agrave aacute acirc atilde auml 
                           aring aelig ccedil egrave eacute ecirc euml 
                           igrave iacute icirc iuml eth ntilde ograve 
                           oacute ocirc otilde ouml divide oslash ugrave 
                           uacute ucirc uuml yacute thorn yuml);
   $upperentities[12] = '#173';

   $self->{'hashentity'} = {};
   for ( my $i=0; $i<=$#upperentities; $i++ )
   {
      my $key = '&'.$upperentities[$i].';';
      $self->{'hashentity'}->{$key}=$i+160;
   }

   $self->{'hashstr'} = (join (';|', @upperentities)).';';

   bless $self, $classname;
   return $self;
}

# version one - for paragraphs
sub pclean
{
   my ($self, $t) = @_;
   return undef if (! defined $t);
   # make ISOlat1 entities into Unicode character entities
   $t =~ s/&($self->{'hashstr'})/sprintf ("&#x%04X;", $self->{'hashentity'}->{$&})/geo;
   # escape non-XML-encoded ampersands (including from other characters sets)
   $t =~ s/&(?!((#[0-9]*)|(#x[0-9]*)|(amp)|(lt)|(gt)|(apos)|(quot));)/&amp;/go;
   # convert extended ascii into Unicode character entities
   $t =~ s/[\xa0-\xff]/'&#'.ord ($&).';'/geo;
   # remove extended ascii that doesnt translate into ISO8859/1
   $t =~ s/[\x00-\x08\x0B\x0C\x0E-\x1f\x80-\x9f]//go;
   # make tags delimiters into entities
   $t =~ s/</&lt;/go;
   $t =~ s/>/&gt;/go;
   # convert any whitespace containing lf or cr into a single cr
   $t =~ s/(\s*[\r\n]\s+)|(\s+[\r\n]\s*)/\n/go;
   # convert multiples spaces/tabs into a single space
   $t =~ s/[ \t]+/ /go;
   # kill leading and terminating spaces
   $t =~ s/^[ ]+(.+)[ ]+$/$1/;
   return $t;
}

# version two - for single-line streams
sub lclean
{
   my ($self, $t) = @_;
   return undef if (! defined $t );
   # make ISOlat1 entities into Unicode character entities
   $t =~ s/&($self->{'hashstr'})/sprintf ("&#x%04X;", $self->{'hashentity'}->{$&})/geo;
   # escape non-XML-encoded ampersands (including from other characters sets)
   $t =~ s/&(?!((#[0-9]*)|(#x[0-9]*)|(amp)|(lt)|(gt)|(apos)|(quot));)/&amp;/go;
   # convert extended ascii into Unicode character entities
   $t =~ s/[\xa0-\xff]/'&#'.ord ($&).';'/geo;
   # remove extended ascii that doesnt translate into ISO8859/1
   $t =~ s/[\x00-\x08\x0B\x0C\x0E-\x1f\x80-\x9f]//go;
   # make tags delimiters into entities
   $t =~ s/</&lt;/go;
   $t =~ s/>/&gt;/go;
   # flatten whitespace
   $t =~ s/[\s\t\r\n]+/ /go;
   # kill leading and terminating spaces
   $t =~ s/^[ ]+(.+)[ ]+$/$1/;
   return $t;
}

sub straighten
{
   my ($self, $t) = @_;
   # eliminate all carriage returns and linefeeds
   $t =~ s/[\t\r\s\n]+/ /go;
   return $t;
}

sub FormatXML
{
   my ($self, $head, $indent) = @_;
   $indent .= $self->{'XMLindent'};
   my ($key, $i, $j, $buffer, @orderedkeys);
   $buffer = '';
   if (exists ($head->{'mdorder'}))
   { @orderedkeys = @{$head->{'mdorder'}}; }
   else
   { @orderedkeys = keys %$head; }
   foreach $key (@orderedkeys)
   {
      if ((exists ($head->{$key})) && (ref ($head->{$key}) eq 'ARRAY'))
      {
         foreach $i (@{$head->{$key}})
         {
            if (ref ($i) eq 'ARRAY')
            {
               my $atthash = $$i[0];
               my $childhash = $$i[1];

               $buffer .= "$indent<$key";
               foreach $j (keys %$atthash)
               {
                  $buffer .= " $j=\"$atthash->{$j}\"";
               }
               $buffer .= ">\n";

               if (ref ($childhash) eq 'HASH')
               {
                  $buffer .= $self->FormatXML ($childhash, $indent);
               }
               else
               {
                  $buffer .= "$indent$childhash\n";
               }

               $buffer .= "$indent</$key>\n";
            }
            elsif (ref ($i) eq 'HASH')
            {
               my $nestedbuffer = $self->FormatXML ($i, $indent);
               if ($nestedbuffer ne '')
               {
                  $buffer .= "$indent<$key>\n$nestedbuffer$indent</$key>\n";
               }
            }
            else
            {
               $buffer .= "$indent<$key>$i</$key>\n";
            }
         }
      }
      elsif ((exists ($head->{$key})) && (ref ($head->{$key}) eq 'HASH'))
      {
         my $nestedbuffer = $self->FormatXML ($head->{$key}, $indent);
         if ($nestedbuffer ne '')
         {
            $buffer .= "$indent<$key>\n$nestedbuffer$indent</$key>\n";
         }
      }
      elsif ((exists ($head->{$key})) && ($head->{$key} ne ''))
      {
         $buffer .= "$indent<$key>$head->{$key}</$key>\n";
      }
   }
   $buffer;
}

sub DateisValid
{
   my ($self, $date) = @_;

   if (!($date =~ /^[0-9]{4}-[0-9]{2}-[0-9]{2}$/))
   { return 0; }
   
   my ($year, $month, $day, $daysinmonth);
   $year = substr ($date, 0, 4);
   $month = substr ($date, 5, 2);
   $day = substr ($date, 8, 2);

   if ((((($year % 4) == 0) && (($year % 100) != 0)) || (($year % 400) == 0))
       && ($month == 2))
   { $daysinmonth = 29; }
   elsif (($month == 4) || ($month == 6) || ($month == 9) || ($month == 11))
   { $daysinmonth = 30; }
   elsif ($month == 2)
   { $daysinmonth = 28; }
   else
   { $daysinmonth = 31; }

   if (($day == 0) || ($day > $daysinmonth))
   { return 0; }
   if (($month == 0) || ($month > 12))
   { return 0; }
   if ($year == 0)
   { return 0; }

   return 1;
}

1;
