#!/usr/bin/perl -w -I.
use strict;

#  ---------------------------------------------------------------------
#   Box editing web interface
#    v1.0
#    April 2002
#  ------------------+--------------------+-----------------------------
#   Hussein Suleman  |   hussein@vt.edu   |    www.husseinsspace.com    
#  ------------------+--------------------+-+---------------------------
#   Department of Computer Science          |        www.cs.vt.edu       
#     Digital Library Research Laboratory   |       www.dlib.vt.edu      
#  -----------------------------------------+-------------+-------------
#   Virginia Polytechnic Institute and State University   |  www.vt.edu  
#  -------------------------------------------------------+-------------


use FindBin;
use lib "$FindBin::Bin/../../lib";

use XOAI::XOAISP;
use Pure::EZCGI;
use Pure::EZXML;
use Pure::X2D;


# display main index of OAI/ODL verbs
sub do_index
{
   my ($cgi, $baseURL) = @_;
   
   print "<p>This is an administrative interface to browse through the contents of an Box archive ".
         "and add/modify/delete metadata records. This is NOT a user interface to the Box - such ".
         "should be provided by a service provider that uses the Box component.</p><hr>";

   print "<p><i>Select a service request to start with (enter parameters if necessary):</i></p>";
   
   print "<form>".
         "<input type=\"hidden\" name=\"action\" value=\"putrecord\">".
         "<input type=\"hidden\" name=\"baseURL\" value=\"$baseURL\">".
         "<input type=\"submit\" value=\"Put Record\">".
         "</form>";
   print "<form>".
         "<input type=\"hidden\" name=\"action\" value=\"listsets\">".
         "<input type=\"hidden\" name=\"baseURL\" value=\"$baseURL\">".
         "<input type=\"submit\" value=\"List Sets\">".
         "</form>";
   print "<form>".
         "<input type=\"submit\" value=\"List Identifiers\">".
         "<input type=\"hidden\" name=\"action\" value=\"listidentifiers\">".
         "<input type=\"hidden\" name=\"baseURL\" value=\"$baseURL\">".
         " from <input type=\"input\" name=\"from\" value=\"".$cgi->param ('from')."\" size=\"10\">".
         " until <input type=\"input\" name=\"until\" value=\"".$cgi->param ('until')."\" size=\"10\">".
         " set <input type=\"input\" name=\"set\" value=\"".$cgi->param ('set')."\" size=\"10\"\">".
         " resTok <input type=\"input\" name=\"resumptionToken\" value=\"".$cgi->param ('resumptionToken')."\" size=\"10\">".
         "</form>";
   print "<form>".
         "<input type=\"submit\" value=\"Get Record\">".
         "<input type=\"hidden\" name=\"action\" value=\"getrecord\">".
         "<input type=\"hidden\" name=\"baseURL\" value=\"$baseURL\">".
         " id <input type=\"input\" name=\"identifier\" value=\"".$cgi->param ('identifier')."\" size=\"10\">".
         " mdp <input type=\"input\" name=\"metadataPrefix\" value=\"".$cgi->param ('metadataPrefix')."\" size=\"10\">".
         "</form>";
}


# list the metadata templates that are available
sub do_listtemplates
{
   opendir (DIR, "$FindBin::Bin/templates");
   my @files = readdir (DIR);
   closedir (DIR);
   
   my $buffer = '';
   
   foreach my $template (grep { /^[^\.]+\.xml$/ } @files)
   {
      my $templatename = $template;
      $templatename =~ s/([^\.]+)\.xml/$1/o;
      
      open (FILE, "$FindBin::Bin/templates/$template");
      my @datalines = <FILE>;
      close (FILE);
      my $data = join ('', @datalines);

      # escape special characters before adding to form
      $data =~ s/[\n\r]/\\n/go;
      $data =~ s/&/&amp;/go;
      $data =~ s/</&lt;/go;
      $data =~ s/"/&quot;/go;
      $data =~ s/'/&apos;/go;
      
      $buffer .=  "[<a href=\"?\" onClick=\"document.edit.metadata.value=\'$data\';document.edit.metadataPrefix.value=\'$templatename\';return false\">Use template for $templatename</a>] ";
   }
   
   $buffer;
}


# create form to add record or add the record to the Box
sub do_putrecord
{
   my ($cgi, $baseURL) = @_;
   
   if (($cgi->param ('metadata') eq '') || ($cgi->param ('metadataPrefix') eq ''))
   {
      if (($cgi->param ('metadata') ne '') || ($cgi->param ('metadataPrefix') ne ''))
      {
         print "<p><i>Missing value for required field</i></p>";
      }
   
      print "<form name=\"edit\" method=\"post\" action=\"?\">".
            "<input type=\"hidden\" name=\"action\" value=\"putrecord\">".
            "<input type=\"hidden\" name=\"baseURL\" value=\"$baseURL\">".
            "<p>identifier (optional) : <input type=\"input\" name=\"identifier\" value=\"".$cgi->param ('identifier')."\"></p>".
            "<p>metadataPrefix : <input type=\"input\" name=\"metadataPrefix\" value=\"".$cgi->param ('metadataPrefix')."\"></p>".
            "<p>sets (optional, comma-separated) : <input type=\"input\" name=\"sets\" value=\"".$cgi->param ('sets')."\"></p>".
            "<p>metadata : <br><textarea name=\"metadata\" wrap=\"virtual\" rows=\"10\" cols=\"60\">".$cgi->param ('metadata')."</textarea></p>".
            '<p>'.do_listtemplates.'</p>'.
            "<p><input type=\"submit\" value=\"Put Record\"></p>".
            "</form>";
   }
   else
   {
      # send request to the Box component
      my $xoai = new XOAI::XOAISP ($baseURL);
      my $response = $xoai->doXOAI ('PutRecord', {
         metadata => $cgi->param ('metadata'),
         metadataPrefix => $cgi->param ('metadataPrefix'),
         identifier => $cgi->param ('identifier'),
         sets => $cgi->param ('sets')
      });
      
      if ($response->code != 200)
      {
         print "Error : ".$response->code." ".$response->message;
      }
      else
      {
         print "OK. <a href=\"?baseURL=$baseURL\">click here to return to front</a>";
      }
   }
}


# list sets that are available on the Box
sub do_listsets
{
   my ($cgi, $baseURL) = @_;
   
   # send request to the Box
   my $xoai = new XOAI::XOAISP ($baseURL);
   my $response = $xoai->doXOAI ('ListSets');
   if ($response->code != 200)
   {
      do_index ($cgi, $baseURL); return;
   }
   
   print "<p><i>List of Sets</i></p>\n";
   
   # parse output and list sets
   my $parser = new Pure::EZXML;
   my $document = $parser->parse ($response->content)->getDocumentElement;
   foreach my $set ($document->getElementsByTagName ('set', 0))
   {
      my $setSpecnodes = $set->getElementsByTagName ('setSpec', 0);
      my $setNamenodes = $set->getElementsByTagName ('setName', 0);
      if (($setSpecnodes->getLength == 1) && ($setNamenodes->getLength == 1))
      {
         my $setSpec = $setSpecnodes->item(0)->getChildNodes->toString;
         my $setName = $setNamenodes->item(0)->getChildNodes->toString;
         print "<p><a href=\"?action=listidentifiers&baseURL=$baseURL&set=$setSpec\">$setName</a></p>\n";
      }
   }
}


# list identifiers (according to given criteria) in the Box
sub do_listidentifiers
{
   my ($cgi, $baseURL) = @_;
   
   # collect parameters for request
   my %parm = ();
   if ($cgi->param ('set') ne '') { $parm{'set'} = $cgi->param ('set'); }
   if ($cgi->param ('from') ne '') { $parm{'from'} = $cgi->param ('from'); }
   if ($cgi->param ('until') ne '') { $parm{'until'} = $cgi->param ('until'); }
   if ($cgi->param ('resumptionToken') ne '') { $parm{'resumptionToken'} = $cgi->param ('resumptionToken'); }

   # send request to the Box
   my $xoai = new XOAI::XOAISP ($baseURL);
   my $response = $xoai->doXOAI ('ListIdentifiers', \%parm );
   if ($response->code != 200)
   {
      do_index ($cgi, $baseURL); return;
   }
   
   print "<p><i>List of Identifiers</i></p>\n";

   # parse output and list identifiers (and resumptionToken)
   my $parser = new Pure::EZXML;
   my $document = $parser->parse ($response->content)->getDocumentElement;
   foreach my $identifiernode ($document->getElementsByTagName ('identifier', 0))
   {
      my $identifier = $identifiernode->getChildNodes->toString;
      my $status = $identifiernode->getAttribute ('status');
      print "<p><b>$identifier</b> ";
      if ($status eq 'deleted')
      {
         print ' <i>deleted</i>';
      }
      else
      {
         print "<a href=\"?action=listmetadataformats&baseURL=$baseURL&identifier=$identifier\">[ListMetadataFormats]</a> ".
               "<a href=\"?action=deleterecord&baseURL=$baseURL&identifier=$identifier\" onClick=\"return confirm (\'Are you sure?\')\">[Delete]</a> ".
               "</p>\n";
      }
   }
   foreach my $resumptionTokennode ($document->getElementsByTagName ('resumptionToken', 0))
   {
      my $resumptionToken = $resumptionTokennode->getChildNodes->toString;
      print "<p>[ Resume from <a href=\"?action=listidentifiers&baseURL=$baseURL&resumptionToken=$resumptionToken\">resumptionToken = $resumptionToken</a> ]</p>"; 
   }
}


# list metadata formats available for a single identifier
sub do_listmetadataformats
{
   my ($cgi, $baseURL) = @_;
   
   # check for identifier
   if ($cgi->param ('identifier') eq '')
   {
      do_index ($cgi, $baseURL); return;
   }

   # send request to the Box
   my $xoai = new XOAI::XOAISP ($baseURL);
   my $response = $xoai->doXOAI ('ListMetadataFormats', { identifier=>$cgi->param ('identifier') } );
   if ($response->code != 200)
   {
      do_index ($cgi, $baseURL); return;
   }
   
   print "<p><i>List of Metadata Formats for ".$cgi->param ('identifier')."</i></p>\n";

   # parse output and list metadata formats   
   my $parser = new Pure::EZXML;
   my $document = $parser->parse ($response->content)->getDocumentElement;
   foreach my $metadataFormat ($document->getElementsByTagName ('metadataFormat', 0))
   {
      my $metadataPrefixnodes = $metadataFormat->getElementsByTagName ('metadataPrefix', 0);
      if ($metadataPrefixnodes->getLength == 1)
      {
         my $metadataPrefix = $metadataPrefixnodes->item(0)->getChildNodes->toString;
         print "<p>Display record in <a href=\"?action=getrecord&baseURL=$baseURL&metadataPrefix=$metadataPrefix&identifier=".$cgi->param ('identifier')."\">$metadataPrefix</a></p>";
      }
   }
}


# get a single record by identifier and metadataPrefix and create form for editing
sub do_getrecord
{
   my ($cgi, $baseURL) = @_;
   
   # check for identifier
   if (($cgi->param ('identifier') eq '') || ($cgi->param ('metadataPrefix') eq ''))
   {
      do_index ($cgi, $baseURL); return;
   }

   # send request to the Box
   my $xoai = new XOAI::XOAISP ($baseURL);
   my $response = $xoai->doXOAI ('GetRecord', { 
      identifier => $cgi->param ('identifier'),
      metadataPrefix => $cgi->param ('metadataPrefix')
   } );
   if ($response->code != 200)
   {
      do_index ($cgi, $baseURL); return;
   }
   
   print "<p><i>Metadata Record : ".$cgi->param ('identifier')." in ".$cgi->param ('metadataPrefix')."</i></p>\n";
   
   # parse output and extract and print record
   my $parser = new Pure::EZXML;
   my $document = $parser->parse ($response->content)->getDocumentElement;
   my $metadata = '';
   foreach my $record ($document->getElementsByTagName ('record', 0))
   {
      # print header information
      my $headernodes = $record->getElementsByTagName ('header', 0);
      if ($headernodes->getLength == 1)
      {
         my $identifiernodes = $headernodes->item(0)->getElementsByTagName ('identifier', 0);
         if ($identifiernodes->getLength == 1)
         {
            print "<p>Identifier : ".$identifiernodes->item(0)->getChildNodes->toString."</p>";
         }
         my $datestampnodes = $headernodes->item(0)->getElementsByTagName ('datestamp', 0);
         if ($datestampnodes->getLength == 1)
         {
            print "<p>Datestamp : ".$datestampnodes->item(0)->getChildNodes->toString."</p>";
         }
      }
      
      # print metadata record
      my $metadatanodes = $record->getElementsByTagName ('metadata', 0);
      if ($metadatanodes->getLength == 1)
      {
         foreach my $mdchild ($metadatanodes->item(0)->getChildNodes)
         {
            if ($mdchild->getNodeType eq ELEMENT_NODE)
            {
               $mdchild->setAttribute ('xsi:schemaLocation', $mdchild->getAttribute ('schemaLocation'));
               $mdchild->removeAttribute ('schemaLocation');
            }
         }
         $metadata = $metadatanodes->item(0)->getChildNodes->toString;
         $metadata =~ s/</&lt;/go;
         print "<p>Metadata : <pre>$metadata</pre></p>";
      }
   }
   
   print "<hr><p><i>Edit Metadata</i></p>";

   print "<form name=\"edit\" method=\"post\" action=\"?\">".
         "<input type=\"hidden\" name=\"action\" value=\"putrecord\">".
         "<input type=\"hidden\" name=\"baseURL\" value=\"$baseURL\">".
         "<input type=\"hidden\" name=\"identifier\" value=\"".$cgi->param ('identifier')."\">".
         "<input type=\"hidden\" name=\"metadataPrefix\" value=\"".$cgi->param ('metadataPrefix')."\">".
         "<p>sets (comma-separated, leave blank to not change) : <input type=\"input\" name=\"sets\" value=\"".$cgi->param ('sets')."\"></p>".
         "<p>metadata : <br><textarea name=\"metadata\" wrap=\"virtual\" rows=\"10\" cols=\"60\">$metadata</textarea></p>".
         '<p>'.do_listtemplates.'</p>'.
         "<p><input type=\"submit\" value=\"Update Record\"></p>".
         "</form>";
}


# delete a record from the Box based on its identifier
sub do_deleterecord
{
   my ($cgi, $baseURL) = @_;
   
   # check for identifier
   if ($cgi->param ('identifier') eq '')
   {
      do_index ($cgi, $baseURL); return;
   }

   # send request to the Box
   my $xoai = new XOAI::XOAISP ($baseURL);
   my $response = $xoai->doXOAI ('PutRecord', { 
      identifier => $cgi->param ('identifier'),
      status => 'deleted'
   } );
   
   do_index ($cgi, $baseURL); return;
}


sub main
{
   my $cgi = new Pure::EZCGI;
   
   # use specified baseURL or calculate relative baseURL
   my $baseURL = '';
   if ($cgi->param ('baseURL') ne '')
   {
      $baseURL = $cgi->param ('baseURL');
   }
   else
   {
      my $requestURI = $ENV{'REQUEST_URI'};
      if ($requestURI =~ /(.*)(?=edit.pl)/)
      { 
         $baseURL = 'http://'.$ENV{'SERVER_NAME'}.':'.$ENV{'SERVER_PORT'}.$1.'box.pl';
      }
   }
   
   # read in access control list
   my $acl = new Pure::X2D ("$FindBin::Bin/edit.xml");
   my $allow = {};
   foreach my $machine ($acl->param ('allow', []))
   {
      my ($a, $b, $c, $d) = unpack ('C4', (gethostbyname ($machine))[4]);
      my $ipaddress = "$a.$b.$c.$d";
      $allow->{$ipaddress} = 1;
   }

   # check access control list
   my $ipaddress = $ENV{'REMOTE_ADDR'}; 
   if (! exists $allow->{$ipaddress})
   { 
      print "Status: 403 Forbidden Operation\n".
            "Content-type: text/plain\n\n".
            "403 : Forbidden Operation\n";
      return;
   }

   # print header
   print "Content-type: text/html\n\n".
         "<html><head><title>Box Archive Editor</title></head>".
         "<body><center><h1>Box Archive Editor</h1><p><font size=\"-1\">baseURL of Box: [$baseURL]</font></p></center>".
         "<hr>";
   
   # dispatch requests
   if ($baseURL eq '')
   {
      print "This script must be co-located with box.pl or baseURL must be supplied as a parameter.\n";
   }
   else
   {
      if ($cgi->param ('action') eq 'putrecord')
      { do_putrecord ($cgi, $baseURL); }
      elsif ($cgi->param ('action') eq 'listsets')
      { do_listsets ($cgi, $baseURL); }
      elsif ($cgi->param ('action') eq 'listidentifiers')
      { do_listidentifiers ($cgi, $baseURL); }
      elsif ($cgi->param ('action') eq 'listmetadataformats')
      { do_listmetadataformats ($cgi, $baseURL); }
      elsif ($cgi->param ('action') eq 'getrecord')
      { do_getrecord ($cgi, $baseURL); }
      elsif ($cgi->param ('action') eq 'deleterecord')
      { do_deleterecord ($cgi, $baseURL); }
      else
      { do_index ($cgi, $baseURL); }
   }
   
   print "<hr><table width=\"100%\"><tr><td align=\"left\"><a href=\"?\">Home<a></td><td align=\"right\"><i>a part of the <a href=\"http://oai.dlib.vt.edu/odl\">Open Digital Libraries</a></i> project</td></tr></table></body></html>";
}

main;
