#  ---------------------------------------------------------------------
#   General database implementation operations for Box
#    v1.1
#    April 2002
#  ------------------+--------------------+-----------------------------
#   Hussein Suleman  |   hussein@vt.edu   |    www.husseinsspace.com    
#  ------------------+--------------------+-+---------------------------
#   Department of Computer Science          |        www.cs.vt.edu       
#     Digital Library Research Laboratory   |       www.dlib.vt.edu      
#  -----------------------------------------+-------------+-------------
#   Virginia Polytechnic Institute and State University   |  www.vt.edu  
#  -------------------------------------------------------+-------------


package ODL::Box::BoxDB;


use DBI;


# open database and create if necessary
sub openDatabase
{
   my ($database, $dbusername, $dbpassword, $table) = @_;
   
   my $dbh = DBI->connect ($database, $dbusername, $dbpassword, { PrintError => 0 });

   # check for existance of index
   my $sth = $dbh->prepare ("select identifier from $table limit 1");
   $sth->execute;
   my @row = $sth->fetchrow_array;
   $sth->finish;
   if ($#row > -1)
   {
      return $dbh;
   }
   
   # make new tables in database
   $sth = $dbh->do ("create table $table (".
                    "identifier varchar(255) not null, ".
                    "datestamp integer not null, ".
                    "status varchar(10), ".
                    "metadataPrefix varchar(100) not null, ".
                    "metadata text, ".
                    "about text, ".
                    "primary key (identifier,metadataPrefix)".
                    ")");

   $sth = $dbh->do ("create index datestamp on $table (datestamp)");
   $sth = $dbh->do ("create index metadataPrefix on $table (metadataPrefix)");

   $sth = $dbh->do ("create table $table"."md (".
                    "metadataPrefix varchar(100) not null, ".
                    "metadataNamespace varchar(255) not null, ".
                    "metadataSchema varchar(255) not null, ".
                    "primary key (metadataPrefix,metadataNamespace)".
                    ")");

   $sth = $dbh->do ("create table $table"."set (".
                    "identifier varchar(255) not null, ".
                    "oaiset varchar(255) not null".
                    ")");

   $sth = $dbh->do ("create index identifier on $table"."set (identifier)");
   $sth = $dbh->do ("create index oaiset on $table"."set (oaiset)");
                    
   $sth = $dbh->do ("create table $table"."admin (".
                    "variable varchar(255) not null, ".
                    "value integer not null, ".
                    "primary key (variable)".
                    ")");
                    
   $sth = $dbh->do ("replace into $table"."admin set variable=\'counter\', value=value+1");

   return $dbh;
}


1;

