#  ---------------------------------------------------------------------
#   Sample harvester sub-class
#    v1.1
#    February 2002
#  ------------------+--------------------+-----------------------------
#   Hussein Suleman  |   hussein@vt.edu   |    www.husseinsspace.com    
#  ------------------+--------------------+-+---------------------------
#   Department of Computer Science          |        www.cs.vt.edu       
#     Digital Library Research Laboratory   |       www.dlib.vt.edu      
#  -----------------------------------------+-------------+-------------
#   Virginia Polytechnic Institute and State University   |  www.vt.edu  
#  -------------------------------------------------------+-------------


package TestHarvester;


use Pure::X2D;
use Pure::EZXML;

use XOAI::Harvester;
use vars ('@ISA');
@ISA = ("XOAI::Harvester");


# constructor
sub new
{
   my ($classname, $configfile) = @_;

   my $self = $classname->SUPER::new ();

   # get configuration from file
   my $con = new Pure::X2D ($configfile);
   $self->{'archive'} = $con->{'archive'};

   bless $self, $classname;
   return $self;
}


# destructor
sub dispose
{
   my ($self) = @_;
}


# pass parameters and archive info to harvester
sub harvest
{
   my ($self, $when, $what, $fromwhere) = @_;
   $self->SUPER::harvest ($self->{'archive'}, $when, $what, $fromwhere);   
}


# "call-back" methods


# start harvesting
sub processHarvestStart
{
   my ($self, $archiveidentifier, $archiveurl, $archiveinterval, 
       $archivemetadataPrefixes, $archiveset, $archiveoverlap,
       $archiveinterrequestgap) = @_;
   # archiveidentifier = string
   # archiveurl = string
   # archiveinterval = interval (seconds)
   # archivemetadataPrefixes = reference to string array
   # archiveset = string
   # archiveoverlap = integer (seconds)
   # archiveinterrequestgap = integer (seconds)
       
   print "Starting to harvest from $archiveidentifier\n";
}


# end harvesting
sub processHarvestEnd
{
   my ($self) = @_;
   print "Finished harvesting\n";
}


# detection of an id
sub processIdStart
{
   my ($self, $identifier, $status) = @_;
   # identifier = string
   # status = string
   
   print "Starting to process $identifier\n";  
}


# end of records for an id
sub processIdEnd
{
   my ($self, $identifier) = @_;
   # identifier = string
   
   print "Finished processing $identifier\n";
}


# process a single record
sub processRecord
{
   my ($self, $xml, $date, $metadataPrefix, $set, $archiveid) = @_;
   # xml = EZXML XML node (uses DOM1 interface)
   # date = integer (UTC datestamp of harvesting)
   # metadataPrefix = string
   # set = string
   # archiveid = string
   
   # get identifier from header of record
   my $identifier = '';
   if ($xml->getElementsByTagName ("header")->getLength == 1)
   {
      my $header = $xml->getElementsByTagName ("header")->item(0);
      if ($header->getElementsByTagName ("identifier")->getLength == 1)
      {
         my $identifiercontainer = $header->getElementsByTagName ("identifier")->item(0);
         if (($identifiercontainer->hasChildNodes) && 
             ($identifiercontainer->getFirstChild->getNodeType == TEXT_NODE))
         {
            $identifier = $identifiercontainer->getFirstChild->getNodeValue;
         }
      }
   }
   $identifier =~ s/^([\s\r\n\t]*)([^\s\r\n\t]*)([\s\r\n\t]*)$/$2/;

   # get metadata chunk from record
   my $metadata = '';
   my $mdnamespace = '';
   my $mdschema = '';
   if ($xml->getElementsByTagName ("metadata")->getLength == 1)
   {
      my $metadatacontainer = $xml->getElementsByTagName ("metadata")->item(0);
      foreach my $metadatachunk ($metadatacontainer->getChildNodes)
      {
         if (($metadata eq '') && ($metadatachunk->getNodeType == ELEMENT_NODE))
         {
            $metadata = $metadatachunk->toString;
            my $mdschemaLocation = $metadatachunk->getAttribute ('schemaLocation');
            my @parts = split (' ', $mdschemaLocation);
            if ($#parts == 1)
            {
               $mdnamespace = $parts[0];
               $mdschema = $parts[1];
            }
         }
      }
   }

   # get about chunk from record
   my $about = '';
   if ($xml->getElementsByTagName ("about")->getLength == 1)
   {
      my $aboutcontainer = $xml->getElementsByTagName ("about")->item(0);
      foreach my $aboutchunk ($aboutcontainer->getChildNodes)
      {
         if (($about eq '') && ($aboutchunk->getNodeType == ELEMENT_NODE))
         {
            $about = $aboutchunk->toString;
         }
      }
   }

   my $datestamp = $self->toISO8601 ($date, 'second');
   
   my $status = $xml->getAttribute ('status');

   # for records that are ok ...
   if (($identifier ne '') && ($date ne '') && ($metadataPrefix ne '') && ($archiveid ne ''))
   {
      print "$identifier $metadataPrefix $datestamp $status\n";
      print "$metadata\n";
      print "$about\n\n";
   }
}


1;

