#  ---------------------------------------------------------------------
#   Stopper for English words
#    v1.01
#    January 2002
#  ------------------+--------------------+-----------------------------
#   Hussein Suleman  |   hussein@vt.edu   |    www.husseinsspace.com    
#  ------------------+--------------------+-+---------------------------
#   Department of Computer Science          |        www.cs.vt.edu       
#     Digital Library Research Laboratory   |       www.dlib.vt.edu      
#  -----------------------------------------+-------------+-------------
#   Virginia Polytechnic Institute and State University   |  www.vt.edu  
#  -------------------------------------------------------+-------------


package IR::Stopper;

sub new 
{
   my ($classname) = @_;
   my $self = {
      stopwords => { qw (
                          a 1 has 1 same 1 about 1 have 1 several 1 
                          among 1 however 1 some 1 all 1 such 1 an 1 
                          and 1 are 1 if 1 as 1 in 1 than 1 at 1 into 1 
                          that 1 is 1 the 1 it 1 their 1 its 1 these 1 
                          be 1 they 1 been 1 this 1 between 1 those 1 both 1 
                          made 1 through 1 but 1 make 1 to 1 by 1 many 1 
                          toward 1 more 1 most 1 must 1 do 1 upon 1 
                          during 1 used 1 using 1 no 1 not 1 each 1 was 1
                          either 1 were 1 of 1 what 1 on 1 which 1 or 1 
                          while 1 for 1 who 1 found 1 will 1 from 1 with 1 
                          further 1 within 1 would 1 i 1
                        )
                   }
   };
   bless $self, $classname;
   return $self;
}

sub stop
{
   my ($self, $aword) = @_;
   
   if (exists $self->{stopwords}->{$aword})
   { return ''; }
   else
   { return $aword; }
}
   
sub test
{
   my ($self) = @_;

   my @words = qw (in hmmmm out and finalize i a wordlist and the);

   foreach my $word (@words)
   {
      print $self->stop ($word)." ";
   }
   print "\n";
}

1;

__END__

=head1 NAME

Stopper - stop word checker

=head1 SYNOPSIS

  use Stopper;
 
  my $stopper = new Stopper;
 
  # perform a few simple tests
  $stopper->test;

  # apply stopper to a single word that is stopped
  my $word = $stopper->stop ('the');
  print "\nthe --> [$word]\n";

  # apply stopper to a single word that is not stopped
  my $word = $stopper->stop ('therewith');
  print "\ntherewith --> [$word]\n";

=head1 DESCRIPTION

This is an information retrieval or search system word stopper
that takes a single word or phrase and converts it to an empty
string if the word is too common.

X<stopper>
X<stopping>
X<stop>
X<information retrieval>
X<search engine>

=head1 METHODS

=over 4

=item stop (word)

Perform stopping on the single word parameter.

=item test

Perform stopping on some simple example text
and output the results to stdout.

=back

=head1 NOTES

This is part of the larger effort to build a trivial search engine
component for componentized digital libraries.

The stoplist was generated from 
http://gateway.library.uiuc.edu/ugl/Bi/Workshop/stopwords.htm.

=head1 CAVEATS

Of course this is untested :)

=head1 BUGS

None that I found yet.

=head1 AUTHOR

Hussein Suleman <hussein@vt.edu>

=head1 HISTORY

This is the first release version.

=cut

