#!/usr/bin/perl

#  ---------------------------------------------------------------------
#   Metadata Editor version 1.0
#    v1.0
#    April 2002
#  ------------------+--------------------+-----------------------------
#   Hussein Suleman  |   hussein@vt.edu   |    www.husseinsspace.com    
#  ------------------+--------------------+-+---------------------------
#   Department of Computer Science          |        www.cs.vt.edu       
#     Digital Library Research Laboratory   |       www.dlib.vt.edu      
#  -----------------------------------------+-------------+-------------
#   Virginia Polytechnic Institute and State University   |  www.vt.edu  
#  -------------------------------------------------------+-------------


use FindBin;
use lib "$FindBin::Bin/lib";

use POSIX;

use Pure::EZXML;
use Pure::EZCGI;
use Pure::MDEdit;


# locations of data and schemas
my $schemaURLPrefix = 'http://someurl/';
my $schemaLocalPrefix = 'schema/';
my $dataPrefix = 'data/';


# create listing of metadata formats and existing entries
sub do_index
{
   # list new resource types
   print "<p><i>This is a list of all metadata schemas available. Click on one of them to create a new record in that format.</i></p>";
   print "<ul>";
   opendir (DIR, $schemaLocalPrefix);
   while ( my $dir = readdir (DIR) )
   {
      if ($dir =~ /^([^\.]+).xsd$/)
      {
         print "<li><a href=\"?action=add&schema=$1.xsd\">$1</a></li>";
      }
   }
   close (DIR);
   print "</ul>";

   # list existing resources
   print "<p><i>This is a list of all the existing XML records. Click on one of them to edit it.</i></p>";
   print "<ul>";
   opendir (DIR, $dataPrefix);
   while ( my $dir = readdir (DIR) )
   {
      if ($dir =~ /^([^\.]+.xml)$/)
      {
         print "<li><a href=\"?action=edit&identifier=$1\">$1</a></li>";
      }
   }
   close (DIR);
   print "</ul>";
   
}


# create a new metadata record
sub do_add
{
   my ($cgi) = @_;

   my $schema = $cgi->param ('schema');

   my $mdedit = new Pure::MDEdit ($schemaLocalPrefix.$schema, { 
      schemaURL => $schemaURLPrefix.$schema,
   });
   
   my $nur = "<h2>New Metadata Record</h2>\n".
             "<center><table bgcolor=\"#dddddd\" width=\"80%\" border=\"0\" cellspacing=\"1\"><tr><td bgcolor=\"#000088\">&nbsp;</td><td align=\"center\"><font size=\"2\"><i>items marked * are mandatory</i></font></td><td bgcolor=\"#000088\">&nbsp;</td><td align=\"center\"><font size=\"2\"><i>click on + to duplicate field</i></font></td></tr></table></center>".
             "<p>";

   if ($cgi->param ('first') eq '')
   {
      print $nur.$mdedit->formFromSchema ({first=>'no', action=>'add', schema=>$schema});
   }
   elsif ($cgi->param ('submit') eq '')
   {
      print $nur.$mdedit->formFromCGI ($cgi, {first=>'no', action=>'add', schema=>$schema});
   }
   else
   {
      if ($mdedit->check ($cgi) > 0)
      {
         print $nur."<center><table bgcolor=\"#dddddd\" width=\"80%\" border=\"0\" cellspacing=\"1\"><tr><td bgcolor=\"#000088\">&nbsp;</td><td align=\"center\"><font size=\"2\"><b>some mandatory fields are empty - please check again</b></font></td></tr></table></center><p>";
         print $mdedit->formFromCGI ($cgi, {first=>'no', action=>'add', schema=>$schema});
      }
      else
      {
         my $xml = $mdedit->XMLFromCGI ($cgi);
         my $parser = new Pure::EZXML;
         my $document = $parser->parse ($xml)->getDocumentElement;

         # quick fudge to fix namespaces that are destroyed by DOM1 parser
         $document->setAttribute ('xmlns:xsi', $document->getAttribute ('xsi'));
         $document->setAttribute ('xsi:schemaLocation', $document->getAttribute ('schemaLocation'));
         $document->removeAttribute ('xsi');
         $document->removeAttribute ('schemaLocation');
         
         # write to new file
         open (FILE, '>'.$dataPrefix.time ().'-'.getpid ().'.xml');
         print FILE $document->toString;
         close (FILE);
         
         do_index;
      }
   }
}


# edit an existing metadata record
sub do_edit
{
   my ($cgi) = @_;
   
   my $nur = "<h2>Edit Metadata Record</h2>\n".
             "<center><table bgcolor=\"#dddddd\" width=\"80%\" border=\"0\" cellspacing=\"1\"><tr><td bgcolor=\"#000088\">&nbsp;</td><td align=\"center\"><font size=\"2\"><i>items marked * are mandatory</i></font></td><td bgcolor=\"#000088\">&nbsp;</td><td align=\"center\"><font size=\"2\"><i>click on + to duplicate field</i></font></td></tr></table></center>".
             "<p>";
   
   if ($cgi->param ('first') eq '')
   {
      # get XML file
      open (FILE, $dataPrefix.$cgi->param ('identifier'));
      my @lines = <FILE>;
      close (FILE);
      my $doc = join ('', @lines);
       
      # parse XML
      my $parser = new Pure::EZXML;
      my $documentroot = $parser->parse ($doc)->getDocumentElement;
      
      # extract original schema
      my $schemaurl = '';
      my $schema = '';
      $schemaurl = $documentroot->getAttribute ('schemaLocation');
      $schemaurl =~ s/([^ ]+) (.*)/$2/go;
      $schema = $schemaurl;
      $schema =~ s/http:\/(\/[^\/]+)*\/(.*)/$2/go;

      my $mdedit = new Pure::MDEdit ($schemaLocalPrefix.$schema, { 
         schemaURL => $schemaURLPrefix.$schema,
      } );

      print $nur.$mdedit->formFromSchema ({first=>'no', action=>'edit', identifier=>$cgi->param ('identifier'), schema=>$schema}, $documentroot);
   }
   else
   {
      my $schema = $cgi->param ('schema');
      
      my $mdedit = new Pure::MDEdit ($schemaLocalPrefix.$schema, { 
         schemaURL => $schemaPrefix.$schema,
      } );
      
      if ($cgi->param ('submit') eq '')
      {
         print $nur.$mdedit->formFromCGI ($cgi, {first=>'no', action=>'edit', identifier=>$cgi->param ('identifier'), schema=>$schema});
      }
      else
      {
         if ($mdedit->check ($cgi) > 0)
         {
            print $nur."<center><table bgcolor=\"#dddddd\" width=\"80%\" border=\"0\" cellspacing=\"1\"><tr><td bgcolor=\"#000088\">&nbsp;</td><td align=\"center\"><font size=\"2\"><b>some mandatory fields are empty - please check again</b></font></td></tr></table></center><p>";
            print $mdedit->formFromCGI ($cgi, {first=>'no', action=>'edit', identifier=>$cgi->param ('identifier'), schema=>$schema});
         }
         else
         {
            my $xml = $mdedit->XMLFromCGI ($cgi);
            my $parser = new Pure::EZXML;
            my $document = $parser->parse ($xml)->getDocumentElement;

            # quick fudge to fix namespaces that are destroyed by DOM1 parser
            $document->setAttribute ('xmlns:xsi', $document->getAttribute ('xsi'));
            $document->setAttribute ('xsi:schemaLocation', $document->getAttribute ('schemaLocation'));
            $document->removeAttribute ('xsi');
            $document->removeAttribute ('schemaLocation');
         
            # write to existing file
            open (FILE, '>'.$dataPrefix.$cgi->param ('identifier'));
            print FILE $document->toString;
            close (FILE);
            
            do_index;
         }
      }
   }
}


# HTML header 
sub header
{
   print "Content-type: text/html\n\n".
         "<html>\n".
         "<head>\n<title>MDEdit Test Interface</title>\n".
         "<style type=\"text/css\">BODY { font-family : sans-serif; font-size : 10pt }</style>".
         "</head>".
         "<body>\n".
         "<h1>MDEdit Test Interface</h1><hr>";
}


# HTML footer
sub footer
{
   print "</body></html>\n";
}


# main program body to get parameters and dispatch as necessary
sub main
{
   my $cgi = new Pure::EZCGI;
   my $action = $cgi->param ('action');
   
   header;
         
   if ($action eq 'add')
   { do_add ($cgi); }
   elsif ($action eq 'edit')
   { do_edit ($cgi); }
   else
   { do_index; }
   
   footer;
}

main;

