#  ---------------------------------------------------------------------
#   File Box
#    v1.0
#    January 2002
#  ------------------+--------------------+-----------------------------
#   Hussein Suleman  |   hussein@vt.edu   |    www.husseinsspace.com    
#  ------------------+--------------------+-+---------------------------
#   Department of Computer Science          |        www.cs.vt.edu       
#     Digital Library Research Laboratory   |       www.dlib.vt.edu      
#  -----------------------------------------+-------------+-------------
#   Virginia Polytechnic Institute and State University   |  www.vt.edu  
#  -------------------------------------------------------+-------------


package Pure::FileBox;


use POSIX;


# constructor
sub new
{
   my ($classname, $locations) = @_;
   
   my $self = {
      map => $locations,
   };
   
   bless $self, $classname;
   return $self;
}


# add a file to the filebox
sub add
{
   my ($self, $filename, $filehandle) = @_;
   
   # strip extra front/back slash characters from filename
   if (rindex ($filename, '\\') != -1)
   {
      $filename = substr ($filename, rindex ($filename, '\\')+1);
   }
   if (rindex ($filename, '/') != -1)
   {
      $filename = substr ($filename, rindex ($filename, '/')+1);
   }   
   
   # select a physical and url location for the file
   my @locationkeys = keys %{$self->{map}};
   if ($#locationkeys == -1)
   { return undef; }
   my $physicallocation = $locationkeys[0];
   my $urllocation = $self->{map}->{$physicallocation};
   if (! defined $urllocation)
   { return undef; }
   
   # create unique prefix
   my $uniqifier = time ().'-'.getpid ();
   
   # create folder
   mkdir "$physicallocation/$uniqifier" || return undef;
   
   # store file
   open (FILE, ">$physicallocation/$uniqifier/$filename") || return undef;
   while (my $data = <$filehandle>)
   {
      print FILE $data;
   }
   close (FILE);

   # return url   
   return "$urllocation/$uniqifier/$filename";
}


# delete a file from the filebox
sub delete
{
   my ($self, $url) = @_;
   
   if ((exists $self->{map}->{donotdelete}) && ($self->{map}->{donotdelete} eq 'true'))
   {
      return;
   }

   my $physicallocation;
   my $uniqifier;
   my $filename;

   # split file into physical location, uniqifier and filename
   my $found = 0;
   foreach my $aphysicallocation (keys %{$self->{map}})
   {
      my $urllocation = $self->{map}->{$aphysicallocation};
      if ($url =~ /$urllocation\/([0-9\-]+)\/(.*)$/)
      {
         $physicallocation = $aphysicallocation; 
         $uniqifier = $1;
         $filename = $2;
         $found = 1;
      }
   }
   
   # if file is not found
   if (($found == 0) || (! -e "$physicallocation/$uniqifier/$filename"))
   { return undef; }
   
   # create deleted folder
   mkdir "$physicallocation/deleted";
   
   # move file+uniqifier to deleted folder
   system "mv","$physicallocation/$uniqifier", "$physicallocation/deleted";
}


# check if the file is local to this filebox or not
sub isLocal
{
   my ($self, $url) = @_;
   
   # split file into physical location, uniqifier and filename
   foreach my $urllocation (values %{$self->{map}})
   {
      if ($url =~ /$urllocation\/([0-9\-]+)\/(.*)$/)
      {
         return 1;
      }
   }
   return 0;
}

1;

