#!/usr/bin/perl

use strict;

#  ---------------------------------------------------------------------
#   simple user interface for WhatsNew component
#    v1.0
#    April 2002
#  ------------------+--------------------+-----------------------------
#   Hussein Suleman  |   hussein@vt.edu   |    www.husseinsspace.com
#  ------------------+--------------------+-+---------------------------
#   Department of Computer Science          |        www.cs.vt.edu
#     Digital Library Research Laboratory   |       www.dlib.vt.edu
#  -----------------------------------------+-------------+-------------
#   Virginia Polytechnic Institute and State University   |  www.vt.edu
#  -------------------------------------------------------+-------------


use FindBin;
use lib "$FindBin::Bin/lib";


use Pure::EZCGI;
use Pure::EZXML;
use XOAI::XOAISP;


# set the baseURL of the WhatsNew engine component
my $baseURL = 'http://oai.dlib.vt.edu/~hussein/cgi-bin/compute/ODL-WhatsNew/WhatsNew/compute/whatsnew.pl';


# generate an HTML prolog and header
sub header
{
   "Content-type: text/html\n\n".
   "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n".
   "<html>\n\n".
   "<head>\n".
   "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=ISO-8859-1\">".
   "<title>COMPUTE Whats New</title>\n".
   "</head>\n\n".
   "<body>\n".
   "<h1><center><font color=\"#88000\">".
   "COMPUTE Whats New\n".
   "</font></center></h1><hr>\n";
}


# generate an HTML footer
sub footer
{
   "<hr><i><center>Send all queries to hussein\@vt.edu</center></i>\n".
   "</body>\n".
   "</html>\n";
}


# format the output from WhatsNew for display, using only the XML::DOM parser
sub Format
{
   my ($data) = @_;

   # parse XML
   my $parser = new Pure::EZXML;
   my $document = $parser->parse ($data)->getDocumentElement;
   
   # mappings from ids to human-readable names and URLs
   my %name = ( 'cogprints' => 'Cogprints', 'CSTC' => 'Computer Science Teaching Center',
                'arXiv' => 'arXiv', 'ncstrlh' => 'NCSTRL Historical Collection' );
   my %URL = ( 'cogprints' => 'http://www.cogprints.org', 'CSTC' => 'http://www.cstc.org',
               'arXiv' => 'http://arXiv.org', 'ncstrlh' => 'http://www.ncstrl.org' );

   # get the total number of hits
   my $hits = $document->getElementsByTagName ('responseContainer', 0)
     ->item(0)->getElementsByTagName ('hits', 0)->item(0)->getChildNodes->toString;
   
   # create output header
   my $buffer = "<p><center><i>Displaying 1-$hits of $hits results</center></p>".
                "<ol>";

   # go through each record in the list
   foreach my $record ($document->getElementsByTagName ('record', 0))
   {
      my @metadata = $record->getElementsByTagName ('metadata', 0);
      if ($#metadata > -1)
      {
         my @dc = $metadata[0]->getElementsByTagName ('dc', 0);
         if ($#dc == -1)
         {
            @dc = $metadata[0]->getElementsByTagName ('oai_dc', 0);
         }
         
         # if a Dublin Core record does exist, get and output fields
         if ($#dc > -1)
         {
            $buffer .= "<li><font size=\"+1\" face=\"arial\">";
            my @title = $dc[0]->getElementsByTagName ('title', 0);
            my @creator = $dc[0]->getElementsByTagName ('creator', 0);
            my @date = $dc[0]->getElementsByTagName ('date', 0);
            my @identifier = $dc[0]->getElementsByTagName ('identifier', 0);
            foreach my $t (@title)
            {
               if ($#identifier > -1)
               {
                  $buffer .= "<a href=\"".$identifier[0]->getChildNodes->toString."\">";
               }
               $buffer .= $t->getChildNodes->toString;
               if ($#identifier > -1)
               {
                  $buffer .= '</a>';
               }
               $buffer .= '<br>';
            }
            $buffer .= "</font><i><font size=\"-1\">";
            my $first = 1;
            if ($#creator > -1)
            {
               $buffer .= 'By : ';
               foreach my $c (@creator)
               {
                  if ($first == 1)
                  { $first = 0; }
                  else
                  { $buffer .= ', '; }
                  my $author = $c->getChildNodes->toString;
                  if ($author =~ /^([^,]+),\s*(.+)$/)
                  {
                     $author = "$2 $1";
                  }
                  $buffer .= $author;
               }
            }
            foreach my $d (@date)
            {
               if ($first == 1)
               { $first = 0; }
               else
               { $buffer .= ', '; }
               $buffer .= ' '.$d->getChildNodes->toString;
            }
            
            # map source from identifier
            my $id = $record->getElementsByTagName ('header')->item(0)
              ->getElementsByTagName ('identifier')->item(0)->getChildNodes->toString;
            if ($id =~ /oai:(cogprints|ncstrlh|CSTC|arXiv)(.*)/)
            {
               $buffer .= " (Source: <a href=\"$URL{$1}\" title=\"OAI identifier: $id\">$name{$1}</a>)";
            }
            $buffer .= '</font></i></li><p>';
         }
      }
   }
   $buffer .= "</ol><center><table width=\"50%\"><tr>";

   $buffer .= '</tr></table></center>';
}


# main program body
sub main
{
   print header;

   print "This is an experimental component to list whats new in a union ".
         "of the NCSTRL-Historical, CogPrints, arXiv-cs and CSTC collections.\n".
         "Reload the page to get a different sample.\n".
         "<hr>";

   # create service provider object
   my $xoai = new XOAI::XOAISP ($baseURL);

   # issue query, format output and print
   my $outputdata = $xoai->doXOAI ('ListRecords', 
      { 
         metadataPrefix => 'oai_dc',
      })->content;
   print Format ($outputdata);

   print footer;
}

main;
