#!/usr/bin/perl

use strict;

#  ---------------------------------------------------------------------
#   simple user interface for Thread component
#    v1.0
#    April 2002
#  ------------------+--------------------+-----------------------------
#   Hussein Suleman  |   hussein@vt.edu   |    www.husseinsspace.com
#  ------------------+--------------------+-+---------------------------
#   Department of Computer Science          |        www.cs.vt.edu
#     Digital Library Research Laboratory   |       www.dlib.vt.edu
#  -----------------------------------------+-------------+-------------
#   Virginia Polytechnic Institute and State University   |  www.vt.edu
#  -------------------------------------------------------+-------------


use FindBin;
use lib ("$FindBin::Bin/lib", "..");


use Pure::EZXML;
use Pure::EZCGI;
use Ext::XSLT;
use XOAI::XOAISP;


# set the baseURL of the Thread component
my $baseURL = 'http://oai.dlib.vt.edu/~odl/cgi-bin/discussion/ODL-Thread-1.0/Thread/odlforum/thread.pl';


# main program body
sub main
{
   my $cgi = new Pure::EZCGI;

   my $identifier = $cgi->param ('identifier');
   my $start = $cgi->param ('start');
   my $username = $cgi->param ('username');
   
   if (($cgi->param ('action') ne '') && ($username ne ''))
   {
      # get datestamp
      my $datestamp = localtime (time);
      
      # get and escape body
      my $body = $cgi->param ('body');
      $body =~ s/&/&amp;/go;
      $body =~ s/</&lt;/go;
      $body =~ s/\n\n/<p\/>/go;
      $body =~ s/\n/<br\/>/go;
      $body =~ s/[\x00-\x08\x0B\x0C\x0E-\x1f\x80-\x9f]//go;
      
      # get and escape subject
      my $subject = $cgi->param ('subject');
      $subject =~ s/&/&amp;/go;
      $subject =~ s/</&lt;/go;
      $subject =~ s/[\x00-\x08\x0B\x0C\x0E-\x1f\x80-\x9f]//go;
      
      # create metadata record
      my $metadata = "<discuss xmlns=\"http://ODLThread\" xsi:schemaLocation=\"http://ODLThread odlthead.xsd\">".
                     "<from>$username</from>".
                     "<datestamp>$datestamp</datestamp>".
                     "<subject>$subject</subject>".
                     "<body>$body</body>".
                     '</discuss>';
      
      # submit record to archive
      my $xoai = new XOAI::XOAISP ($baseURL);
      my $response = $xoai->doXOAI ('PutRecord', {
         metadata => $metadata,
         metadataPrefix => 'annotation', 
         sets => $cgi->param ('set'),
      });
   
      # get identifier for new record
      if ($response->code == 200)
      {
         my $parser = new Pure::EZXML;
         my $document = $parser->parse ($response->content)->getDocumentElement;
         my $identifiernode = $document->getElementsByTagName ('identifier', 0);
         if ($identifiernode->getLength == 1)
         {
            $identifier = $identifiernode->item(0)->getChildNodes->toString;
            $start = 1;
         }
      }
   }
   
   # run identifier through stylesheet to create output
   my $xslt = new Ext::XSLT ('forum.xsl');
   print "Content-type: text/html\n\n";
   print $xslt->Transform ("<forum><baseURL>$baseURL</baseURL><identifier>$identifier</identifier><start>$start</start></forum>");
}

main;
