//  ----------------------------------------------------------------------
// | Open Digital Libraries Component Explorer - version 1.0              |
// | Hussein Suleman                                                      |
// | September 2002                                                       |
//  ----------------------------------------------------------------------
// |  Virginia Polytechnic Institute and State University                 |
// |  Department of Computer Science                                      |
// |  Digital Library Research Laboratory                                 |
//  ----------------------------------------------------------------------

// ======================================================================
//  Module     : getproto
//  Purpose    : get the full protocol support list for a component
// ======================================================================

#include <stdio.h>
#include <stdlib.h>
#include <fstream.h>
#include <unistd.h>
#include <string.h>

#include "config.h"
#include "taglist.h"
#include "xml.h"


// ----------------------------------------------------------------------
//  Function   : GetProtocol
// ----------------------------------------------------------------------
//  Purpose    : get full list of protocol versions for component
//  Parameters : archive baseURL
//  Result     : string of protocol versions
// ----------------------------------------------------------------------

char * GetProtocol ( char *archiveurl )
{
   char *Identifyfilename;
   char Identifybuffer[L_tmpnam];
   char errorfilename[L_tmpnam];
   char oaiurl[2048], lynxcommand[2048];
   TagList tl;
   static char protocolVersion[1024];
   char repositoryName[1024];
   
   strcpy (repositoryName, "");
   strcpy (protocolVersion, "");

   tmpnam (Identifybuffer);
   tmpnam (errorfilename);
   Identifyfilename = Identifybuffer;

   sprintf (oaiurl, "%s?verb=Identify", archiveurl);
   sprintf (lynxcommand, "%s -error_file=%s -source \"%s\" > %s", lynxpath, errorfilename, oaiurl, Identifyfilename);
   system (lynxcommand);
      
   // check for HTTP errors
   char linebuffer[2048];
   int statuscode;
   fstream errorfile (errorfilename, ios::in);
   errorfile.getline (linebuffer, 2048);
   errorfile >> linebuffer >> statuscode;
   errorfile.getline (linebuffer, 2048);

   unlink (errorfilename);

   if (statuscode >= 400)
   {
      unlink (Identifyfilename);
      return protocolVersion;
   }
   
   if (runparser (&tl, Identifyfilename))
   {
      TagList *p = tl.head;

      // cater for OAI-PMH v2.0 encapsulation
      p = tl.Search ("OAI-PMH");
      if (p)
         p = p->Search ("Identify");
      else
         p = tl.head;

      if (p)
         p = p->head;

      while (p != NULL)
      {
         // get repository name
         if ((p->type == TAG) && (strcmp (p->tag, "repositoryName") == 0))
         {
            if ((p->head) && (p->head->type == TEXT))
               strcat (repositoryName, p->head->tag);
         }
         
         // get OAI protocol version
         if ((p->type == TAG) && (strcmp (p->tag, "protocolVersion") == 0))
         {
            if ((p->head) && (p->head->type == TEXT))
            {
               strcat (protocolVersion, "[OAI:");
               strcat (protocolVersion, p->head->tag);
               strcat (protocolVersion, "]");
            }
         }
         
         // get ODL protocol versions
         if ((p->type == TAG) && (strcmp (p->tag, "description") == 0))
         {
            TagList *r = p->Search ("odl-description");
            if (r)
            {
               TagList *q = r->Search ("protocol");
               strcat (protocolVersion, "[");
               if ((q) && (q->head) && (q->head->type == TEXT))
                  strcat (protocolVersion, q->head->tag);
               strcat (protocolVersion, ":");
               q = r->Search ("version");
               if ((q) && (q->head) && (q->head->type == TEXT))
                  strcat (protocolVersion, q->head->tag);
               strcat (protocolVersion, "]");
            }
         }
         p = p->next;
      }
   }

   unlink (Identifyfilename);
   
   strcat (protocolVersion, " ");
   strcat (protocolVersion, repositoryName);
   
   return protocolVersion;
}

