//  ----------------------------------------------------------------------
// | Open Digital Libraries Component Explorer - version 1.0              |
// | Hussein Suleman                                                      |
// | September 2002                                                       |
//  ----------------------------------------------------------------------
// |  Virginia Polytechnic Institute and State University                 |
// |  Department of Computer Science                                      |
// |  Digital Library Research Laboratory                                 |
//  ----------------------------------------------------------------------

// ======================================================================
//  Module     : testodl
//  Purpose    : Main program
// ======================================================================

#include <dirent.h>
#include <fstream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <sys/types.h>

#include "blocks.h"
#include "cgi.h"
#include "config.h"
#include "escape.h"
#include "language.h"
#include "parser.h"
#include "parser2.h"
#include "parserodl.h"
#include "sortlist.h"
#include "taglist.h"
#include "validate.h"
#include "xml.h"

// ----------------------------------------------------------------------
//  Function   : GetTimeString
// ----------------------------------------------------------------------
//  Purpose    : Get current time/date in string format
//  Parameters : (none)
//  Result     : Time/date string
// ----------------------------------------------------------------------

char *GetTimeString ()
{
   static char nowstr[80];
   time_t tt;
   time (&tt);
   strcpy (nowstr, ctime (&tt));
   nowstr[strlen (nowstr)-1] = 0;
   return nowstr;
}

// ----------------------------------------------------------------------
//  Function   : Log
// ----------------------------------------------------------------------
//  Purpose    : Log activity with time/date stamp
//  Parameters : String to log
//  Result     : (none)
// ----------------------------------------------------------------------

void Log ( char *s )
{
   fstream f ("userlog", ios::app);
   f << GetTimeString () << " : ";
   f << getenv ("REMOTE_ADDR") << " : ";
   f << s << "\n";
}

// ----------------------------------------------------------------------
//  Function   : OutputArchiveSelector
// ----------------------------------------------------------------------
//  Purpose    : Display archive input box and preselecteds
//  Parameters : 1. language name
//               2. bgcolor
//               3. headercolor
//               4. blockcolor
//  Result     : (none)
// ----------------------------------------------------------------------

void OutputArchiveSelector ( char *language, char *bgcolor, 
                             char *headercolor, char *blockcolor )
{
   cout << "<center>"
        << "<p>" << Translate ("This site presents an interface to interactively test archives for compliance with the OAI Protocol for Metadata Harvesting")
        << " [<a href=\"http://www.openarchives.org/OAI/2.0/openarchivesprotocol.htm\">"
        << Translate ("Click here for details") << "</a>]"
        << Translate (" or the Open Digital Library protocols")
        << " [<a href=\"http://oai.dlib.vt.edu/odl/\">"
        << Translate ("Click here for details") << "</a>]"
        << "</p><p>" << Translate ("JavaScript is required")
        << "</p><p>" << Translate ("Note: To avoid HTTP errors, please wait for each page to finish loading before clicking on any link.")
        << "</p></center><hr>";

   cout << "<p>" << Translate ("Please enter the baseURL to the OAI/ODL interface (everything before the ?) or choose a predefined component from the table")
        << "<br>" << Translate ("Then click on a verb from the list below to test that function (entering parameters as necesary)")
        << "</p>"
        << Translate ("baseURL") << " : <input type=text size=80 name=archive>";
      
   char aname[1024];
   char aurl[1024];
   char adesc[1024];
   fstream *predef;

   // create stored list of archives in javascript
   predef = new fstream ("predef", ios::in);
   SortList sl;
   cout << "<script>\n"
        << "var namelist = new Array();\n"
        << "var urllist = new Array();\n";
   while (!(predef->eof ()))
   {
      predef->getline (aname, sizeof (aname));
      predef->getline (aurl, sizeof (aurl));
      if ((strlen (aname) > 0) && (strlen (aurl) > 0))
         sl.Add (aname, aurl, "");
   }
   
   SortNode *sn = sl.head;
   while (sn != NULL)   
   {
      cout << "namelist[namelist.length] = \"" << sn->name << "\";\n"
           << "urllist[urllist.length] = \"" << sn->url << "\";\n";
      sn = sn->next;
   }
   
   // javascript functions to update form
   cout << "\nfunction UpdateArchive ()\n"
        << "{\n"
        << "   document.mainform.archive.value = urllist[document.mainform.archivelist.selectedIndex];\n"
        << "}\n";
   cout << "</script>";
      
   delete predef;

   // output selection list of predefined archives
   cout << "<p><select name=archivelist size=4 onChange=\""
        << "UpdateArchive ()\">";
   sn = sl.head;
   while (sn != NULL)
   {
      cout << "<option value=\"" << sn->url << "\">" << sn->name << "</option>";
      sn = sn->next;
   }
   cout << "</select>";
   
   if (language == NULL)
      language = "";
   cout << "<p>"
        << "[ <a href=\"addarchive?language=" << language
        << "&bgcolor=" << bgcolor
        << "&headercolor=" << headercolor
        << "&blockcolor=" << blockcolor
        << "\">"
        << Translate ("Test and Add a component to this list")
        << "</a> ]</p>";
}

// ----------------------------------------------------------------------
//  Function   : OutputHeadersAndForm
// ----------------------------------------------------------------------
//  Purpose    : Output headers for page
//  Parameters : 1. protocol version
//               2. archive URL
//               3. language name
//               4. bgcolor
//               5. headercolor
//               6. blockcolor
//               7. URL to display
//               8. descriptive URL for archive
//               9. verb
//  Result     : (none)
// ----------------------------------------------------------------------

void OutputHeadersAndForm ( char *protocolVersion, char *archive, char *language, 
                            char *bgcolor, char *headercolor, char *blockcolor, 
                            char *url = NULL, char *description = NULL, 
                            char *verb = NULL )
{
   OutputHeaders (protocolVersion, bgcolor, headercolor);
   
   // output url and description line
   if ((url) && (strcmp (url, "") != 0))
   {
      cout << "<h4><center>";
      cout << url;
      cout << "</center></h4>";
   
      if ((description) && (strcmp (description, "") != 0))
      {
         cout << "<h5><center>";
         cout << Translate ("Archive details");
         cout << " : <a href=\"";
         cout << description << "\">" << description << "</a>";
         cout << "</center></h5>";
      }
      
      cout << "<hr>";
   }

   // create hidden form and archive input section if necessary
   cout << "<form name=\"mainform\" method=\"post\" action=\"" << testodl << "\">";
   
   if ((archive) && (strcmp (archive, "") != 0))
   { 
      cout << "<input type=hidden name=archive value=\"";
      cout << archive;
      cout << "\">";
      
      cout << "<script>\n";
      
      // function to erase selective parameters
      cout << "function ZapParameters (mask)\n"
           << "{\n"
           << "   if ((mask & 1) == 1) document.mainform.from.value = \'\';\n"
           << "   if ((mask & 2) == 2) document.mainform.until.value = \'\';\n"
           << "   if ((mask & 4) == 4) document.mainform.set.value = \'\';\n"
           << "   if ((mask & 8) == 8) document.mainform.metadataPrefix.value = \'\';\n"
           << "   if ((mask & 16) == 16) document.mainform.identifier.value = \'\';\n"
           << "   if ((mask & 32) == 32) document.mainform.resumptionToken.value = \'\';\n"
//           << "   if ((mask & 64) == 64) document.mainform.metadata.value = \'\';\n"
//           << "   if ((mask & 128) == 128) document.mainform.sets.value = \'\';\n"
           << "}\n";

      // function to fill in resumptionToken forms
      cout << "function FillInRT (averb,atoken)\n"
           << "{\n"
           << "   ZapParameters (31);\n"
           << "   document.mainform.verb.value = averb;\n"
           << "   document.mainform.resumptionToken.value = atoken;\n"
           << "   document.mainform.submit ();\n"
           << "}\n";

      // function to fill in ListIdentifiers forms
      cout << "function FillInLI (aset)\n"
           << "{\n"
           << "   ZapParameters (56);\n"
           << "   document.mainform.verb.value = \'ListIdentifiers\';\n"
           << "   document.mainform.set.value = aset;\n"
           << "   document.mainform.metadataPrefix.value = \'oai_dc\';\n"
           << "   document.mainform.submit ();\n"
           << "}\n";

      // function to fill in GetRecord forms
      cout << "function FillInGR (ametadataPrefix,anidentifier)\n"
           << "{\n"
           << "   ZapParameters (39);\n"
           << "   document.mainform.verb.value = \'GetRecord\';\n"
           << "   document.mainform.metadataPrefix.value = ametadataPrefix;\n"
           << "   document.mainform.identifier.value = anidentifier;\n"
           << "   document.mainform.submit ();\n"
           << "}\n";

      // function to fill in ListMetadataFormats forms
      cout << "function FillInLMF (anidentifier)\n"
           << "{\n"
           << "   ZapParameters (47);\n"
           << "   document.mainform.verb.value = \'ListMetadataFormats\';\n"
           << "   document.mainform.identifier.value = anidentifier;\n"
           << "   document.mainform.submit ();\n"
           << "}\n";
           
      cout << "</script>\n";
   }
   else
      OutputArchiveSelector (language, bgcolor, headercolor, blockcolor);
      
   cout << "<input type=hidden name=description value=\"";
   if (description)
      cout << description;
   cout << "\">";
   
   cout << "<input type=hidden name=verb value=\"";
   if (verb)
      cout << verb;
   cout << "\">";
   
   cout << "<input type=hidden name=bgcolor value=\"" << bgcolor << "\">";
   cout << "<input type=hidden name=headercolor value=\"" << headercolor << "\">";
   cout << "<input type=hidden name=blockcolor value=\"" << blockcolor << "\">";
   
   cout.flush ();
}

// ----------------------------------------------------------------------
//  Function   : OutputMenu
// ----------------------------------------------------------------------
//  Purpose    : Output verb/options menu
//  Parameters : 1. language name
//               2. headercolor
//               3. blockcolor
//               4. display type (raw/parsed/both)
//               5. validation type (none/local/online)
//               from/until/metadataPrefix/identifier/set/metadata/sets/
//               resumptionToken
//               x. protocol version
//  Result     : (none)
// ----------------------------------------------------------------------

void OutputMenu ( char *language, char *headercolor, char *blockcolor,
                  char *display, char *validation, char *from, char *until, 
                  char *metadataPrefix, char *identifier, char *set,
                  char *resumptionToken, char *metadata, char *sets,
                  char *protocolVersion )
{
   cout << "<center><table width=\"100%\" border=0 cellspacing=5 cellpadding=5>"
        << "<tr bgcolor=\"#" << headercolor <<  "\"><th>" << Translate ("Verbs")
        << "</th><th colspan=2>" << Translate ("Parameters") << "</th></tr>";

   // output protocol version
   if (protocolVersion)
      cout << "<input type=hidden name=\"protocolVersion\" value=\""
           << protocolVersion << "\">";
 
   // output verb links  
   cout << "<tr bgcolor=\"#" << blockcolor << "\"><td>"
        << "<a href=\"\" onClick=\"document.mainform.verb.value=\'Identify\'; document.mainform.submit (); return false\">"
        << "Identify" << "</a><br>"
        << "<a href=\"\" onClick=\"document.mainform.verb.value=\'ListMetadataFormats\'; document.mainform.submit (); return false\">"
        << "List Metadata Formats" << "</a><br>"
        << "<a href=\"\" onClick=\"document.mainform.verb.value=\'ListSets\'; document.mainform.submit (); return false\">"
        << "List Sets" << "</a><br>"
        << "<a href=\"\" onClick=\"document.mainform.verb.value=\'ListIdentifiers\'; document.mainform.submit (); return false\">"
        << "List Identifiers" << "</a><br>"
        << "<a href=\"\" onClick=\"document.mainform.verb.value=\'ListRecords\';"
        << "document.mainform.submit (); return false\">"
        << "List Records" << "</a><br>"
        << "<a href=\"\" onClick=\"document.mainform.verb.value=\'GetRecord\';"
        << "document.mainform.submit (); return false\">"
        << "Get Record" << "</a><br>"
//        << "<a href=\"\" onClick=\"document.mainform.verb.value=\'PutRecord\';"
//        << "document.mainform.submit (); return false\">"
//        << "Put Record" << "</a>"
        << "</td>";
   
   // output parameter fields
   cout << "<td colspan=2><table>";

   cout << "<tr><td align=right>" 
        << "from (eg., YYYY-MM-DD)" << "</td><td>:</td><td>"
        << "<input type=text name=from size=20 maxlength=80 value=\"";
   if (from)
      cout << from;
   cout << "\"></td></tr>";
        
   cout << "<tr><td align=right>" 
        << "until (eg., YYYY-MM-DD)" << "</td><td>:</td><td>"
        << "<input type=text name=until size=20 maxlength=80 value=\"";
   if (until)
      cout << until;     
   cout << "\"></td></tr>";
        
   cout << "<tr><td align=right>" 
        << "metadataPrefix" << "</td><td>:</td><td>"
        << "<input type=text name=metadataPrefix size=15 maxlength=80 value=\"";
   if (metadataPrefix)
      cout << metadataPrefix;
   cout << "\"></td></tr>";

   cout << "<tr><td align=right>"
        << "identifier" << "</td><td>:</td><td>"
        << "<input type=text name=identifier size=20 maxlength=80 value=\"";
   if (identifier)
      cout << identifier;
   cout << "\"></td></tr>";
                
   cout << "<tr><td align=right>" 
        << "set" << "</td><td>:</td><td>"
        << "<input type=text name=set size=20 maxlength=80 value=\"";
   if (set)
      cout << set;
   cout << "\"></td></tr>";
         
   cout << "<tr><td align=right>" 
        << "resumptionToken" << "</td><td>:</td><td>"
        << "<input type=text name=resumptionToken size=20 maxlength=1024 value=\"";
   if (resumptionToken)
      cout << resumptionToken;
   cout << "\"></td></tr>";

//   cout << "<tr><td align=right>" 
//        << "metadata" << "</td><td>:</td><td>"
//        << "<textarea type=text name=metadata wrap=virtual rows=1 columns=20>";
//   if (metadata)
//      cout << metadata;
//   cout << "</textarea></td></tr>";

//   cout << "<tr><td align=right>" 
//        << "sets" << "</td><td>:</td><td>"
//        << "<input type=text name=sets size=20 maxlength=1024 value=\"";
//   if (sets)
//      cout << sets;
//   cout << "\"></td></tr>";

   cout << "</table></td></tr>";

   // output options fields   
   cout << "<tr bgcolor=\"#" << headercolor << "\"><th>" 
        << Translate ("Language") << "</th><th>" << Translate ("Display") 
        << "</th><th>" << Translate ("Schema Validation") << "</th></tr>";
        
   cout << "<tr bgcolor=\"#" << blockcolor << "\">";

   cout << "<td><select name=\"language\" onChange=\"document.mainform.submit (); return false\">";
   DIR *aDir = opendir (".");
   struct dirent *sd;
   if (aDir)
   {
      while (sd = readdir (aDir))
      {
         int sdl = strlen (sd->d_name)-4;
         if ((sd->d_name[sdl] == '.') &&
             (sd->d_name[sdl+1] == 'l') &&
             (sd->d_name[sdl+2] == 'a') &&
             (sd->d_name[sdl+3] == 'n'))
         {                            
            cout << "<option value=\"" << sd->d_name << "\"";
            if ((language != NULL) && (strcmp (sd->d_name, language) == 0))
               cout << " selected";
            cout << ">";
            fstream sdf (sd->d_name, ios::in);
            char languagename[1024];
            sdf >> languagename;
            cout << languagename << "</option>";
         }
      }
      closedir (aDir);
   }
   cout << "</select></td>";
   
   cout << "<td>";
   cout << "<input type=radio name=display value=\"parsed\"";
   if ((display != NULL) && (strcmp (display, "parsed") == 0))
      cout << " checked";
   cout << ">" << Translate ("Parsed");
   cout << "<br><input type=radio name=display value=\"raw\"";
   if ((display != NULL) && (strcmp (display, "raw") == 0))
      cout << "checked";
   cout << ">" << Translate ("Raw XML");
   cout << "<br><input type=radio name=display value=\"both\"";
   if ((display != NULL) && (strcmp (display, "both") == 0))
      cout << "checked";
   cout << ">" << Translate ("Both");
   cout << "</td>";

   cout << "<td>";
   cout << "<input type=radio name=validation value=\"none\"";
   if ((validation != NULL) && (strcmp (validation, "none") == 0))
      cout << " checked";
   cout << ">" << Translate ("None");
   cout << "<br><input type=radio name=validation value=\"localxerces\"";
   if ((validation != NULL) && (strcmp (validation, "localxerces") == 0))
      cout << "checked";
   cout << ">" << Translate ("Local mirror of schemata") << " (Xerces)";
   cout << "<br><input type=radio name=validation value=\"onlinexerces\"";
   if ((validation != NULL) && (strcmp (validation, "onlinexerces") == 0))
      cout << "checked";
   cout << ">" << Translate ("Online schemata") << " (Xerces)";
   cout << "<br><input type=radio name=validation value=\"localxsv\"";
   if ((validation != NULL) && (strcmp (validation, "localxsv") == 0))
      cout << "checked";
   cout << ">" << Translate ("Local mirror of schemata") << " (XSV)";
   cout << "<br><input type=radio name=validation value=\"onlinexsv\"";
   if ((validation != NULL) && (strcmp (validation, "onlinexsv") == 0))
      cout << "checked";
   cout << ">" << Translate ("Online schemata") << " (XSV)";
   cout << "</td>";
   
   cout << "</tr>";

   cout << "</table></center>";
   
   cout << "</form>";
}

// ----------------------------------------------------------------------
//  Function   : DumpFile
// ----------------------------------------------------------------------
//  Purpose    : Outputs a raw XML file
//  Parameters : filename
//  Result     : None
// ----------------------------------------------------------------------

void DumpFile ( char *filename )
{
   FILE *rawf = fopen (filename, "r");
   char achar;
   cout << "<pre>";
   while (!feof (rawf))
   {
      achar = getc (rawf);
      if (!feof (rawf)) 
      {
         if (achar == '<')
            cout << "&lt;";
         else if (achar == '>')
            cout << "&gt;";
         else if (achar == '&')
            cout << "&amp;";
         else
            cout << achar;
      }
   }
   cout << "</pre>";
   fclose (rawf);
}

// ----------------------------------------------------------------------
//  Function   : AddParameter
// ----------------------------------------------------------------------
//  Purpose    : Adds a non-empty parameter to a growing string list
//  Parameters : 1. parameter list string
//               2. string to add on
//               3. name of parameter being added
//  Result     : None
// ----------------------------------------------------------------------

void AddParameter ( char *parameters, char *value, char *name )
{
   if ((value) && (strlen (value) > 0))
   {
      strcat (parameters, "&");
      strcat (parameters, name);
      strcat (parameters, "=");
      strcat (parameters, value);
   }
}


char *getProtocolVersion ( char *verb, char *datafilename, char *archive )
{
   char *Identifyfilename;
   char Identifybuffer[L_tmpnam];
   char errorfilename[L_tmpnam];
   char oaiurl[2048], lynxcommand[2048];
   TagList tl;
   static char protocolVersion[32];

   if (strcmp (verb, "Identify") == 0)
   {
      Identifyfilename = datafilename;
   }
   else
   {
      tmpnam (Identifybuffer);
      tmpnam (errorfilename);
      Identifyfilename = Identifybuffer;

      sprintf (oaiurl, "%s?verb=Identify", archive);
      sprintf (lynxcommand, "%s -error_file=%s -source \"%s\" > %s", lynxpath, errorfilename, oaiurl, Identifyfilename);
      Log (oaiurl);
      system (lynxcommand);
      
      // check for HTTP errors
      char linebuffer[2048];
      int statuscode;
      fstream errorfile (errorfilename, ios::in);
      errorfile.getline (linebuffer, 2048);
      errorfile >> linebuffer >> statuscode;
      errorfile.getline (linebuffer, 2048);
      
      unlink (errorfilename);

      if (statuscode >= 400)
      {
         unlink (Identifyfilename);
         strcpy (protocolVersion, reprotocolversion);
         return protocolVersion;
      }
   }
   
   if (runparser (&tl, Identifyfilename))
   {
//      TagList *p = tl.head->Search ("protocolVersion");
//      if ((p) && (p->head) && (p->head->type == TEXT))
//         strcpy (protocolVersion, p->head->tag);
//      p = tl.head->Search ("Identify");
//      if (p)
//      {
//         TagList *q = p->Search ("protocolVersion");
//         if ((q) && (q->head) && (q->head->type == TEXT))
//            strcpy (protocolVersion, q->head->tag);
//      }

      TagList *p = tl.head;
         
      // cater for OAI-PMH v2.0 encapsulation
      p = tl.Search ("OAI-PMH");
      if (p)
         p = p->Search ("Identify");
      else
         p = tl.head;
                                                   
      if (p)
         p = p->head;
                                                                  
      while (p != NULL)
      {
         // get OAI protocol version
         if ((p->type == TAG) && (strcmp (p->tag, "protocolVersion") == 0))
         {
            if ((p->head) && (p->head->type == TEXT))
            {
               strcpy (protocolVersion, p->head->tag);
            }
         }
                                                                                                                                                                                                                                                                  
         // get ODL protocol versions
         if ((p->type == TAG) && (strcmp (p->tag, "description") == 0))
         {
            TagList *r = p->Search ("odl-description");
            if (r)
            {
               strcpy (protocolVersion, "odl");
            }
         }
         
         p = p->next;
      }
   }
   else
   {
      strcpy (protocolVersion, reprotocolversion);
   }

   if (strcmp (verb, "Identify") != 0)
   {
      unlink (Identifyfilename);
   }
   
   return protocolVersion;
}


// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int main ()
{
   // CGI parameters for protocol and options
   CGIProcessor cgi;
   char *archive = cgi.ParmList.Search ("archive");   
   char *verb = cgi.ParmList.Search ("verb");
   char *description = cgi.ParmList.Search ("description");
   char *identifier = cgi.ParmList.Search ("identifier");
   char *metadataPrefix = cgi.ParmList.Search ("metadataPrefix");
   char *set = cgi.ParmList.Search ("set");
   char *from = cgi.ParmList.Search ("from");
   char *until = cgi.ParmList.Search ("until");
   char *resumptionToken = cgi.ParmList.Search ("resumptionToken");
   char *metadata = cgi.ParmList.Search ("metadata");
   char *sets = cgi.ParmList.Search ("sets");
   char *display = cgi.ParmList.Search ("display");
   char *validation = cgi.ParmList.Search ("validation");
   char *language = cgi.ParmList.Search ("language");
   char *bgcolor = cgi.ParmList.Search ("bgcolor");
   char *headercolor = cgi.ParmList.Search ("headercolor");
   char *blockcolor = cgi.ParmList.Search ("blockcolor");
   char *protocolVersion = cgi.ParmList.Search ("protocolVersion");

   // set options defaults
   if ((!display) || (strlen (display) == 0))
      display = "parsed";
   if ((!validation) || (strlen (validation) == 0))
      validation = "localxerces";
   if ((!language) || (strlen (language) == 0))
      language = "enus.lan";
   if ((!bgcolor) || (strlen (bgcolor) == 0))
      bgcolor = bgcolorpreset;
   if ((!headercolor) || (strlen (headercolor) == 0))
      headercolor = headercolorpreset;
   if ((!blockcolor) || (strlen (blockcolor) == 0))
      blockcolor = blockcolorpreset;
   if (!protocolVersion)
      protocolVersion = "";
 
   // set language from parameter
   SetLanguage (language);

   // temporary filenames for output from system calls
   char datafilename[L_tmpnam];
   char errorfilename[L_tmpnam];
   char xsvfilename[L_tmpnam];
   tmpnam (datafilename);
   tmpnam (errorfilename);
   tmpnam (xsvfilename);

   // buffers for commands and XML tree
   char lynxcommand[2048], oaiurl[2048];
   TagList tl;
   
   // output headers, depending on whether or not an archive has been selected
   if ((archive) && (strlen (archive) > 0) && (verb) && (strlen (verb) > 0))
   {
      char parameters[2048];
      parameters[0] = 0;
      AddParameter (parameters, from, "from");
      AddParameter (parameters, until, "until");
      AddParameter (parameters, metadataPrefix, "metadataPrefix");
      AddParameter (parameters, set, "set");
      AddParameter (parameters, identifier, "identifier");
      AddParameter (parameters, resumptionToken, "resumptionToken");
//      AddParameter (parameters, metadata, "metadata");
//      AddParameter (parameters, sets, "sets");
      sprintf (oaiurl, "%s?verb=%s%s", archive, verb, parameters);
      sprintf (lynxcommand, "%s -error_file=%s -source \"%s\" > %s", lynxpath, errorfilename, oaiurl, datafilename);
      Log (oaiurl);
      OutputHeadersAndForm (protocolVersion, archive, language, bgcolor, headercolor, 
                            blockcolor, oaiurl, description, verb);

   fstream f ("debug.txt", ios::app);
   f << parameters << "\n";

   }
   else
      OutputHeadersAndForm (protocolVersion, archive, language, bgcolor, headercolor, 
                            blockcolor);

   if ((archive) && (strlen (archive) > 0) && (verb) && (strlen (verb) > 0))
   {
      // send HTTP request to OAI server
      system (lynxcommand);
      
      // check for HTTP errors
      char linebuffer[2048];
      int statuscode;
      fstream errorfile (errorfilename, ios::in);
      errorfile.getline (linebuffer, 2048);
      errorfile >> linebuffer >> statuscode;
      errorfile.getline (linebuffer, 2048);

      if (statuscode >= 400)
      {
         cout << Translate ("HTTP Status Code") << " : " << statuscode 
              << "<p>" << Translate ("HTTP Error Message") 
              << " : " << linebuffer << "\n";
         DumpFile (datafilename);
      }
      
      // if no HTTP errors ...
      else
      {
         if (((!protocolVersion) || (strlen (protocolVersion) == 0)) &&
             ((strcmp (validation, "localxerces") == 0) ||
              (strcmp (validation, "onlinexerces") == 0) ||
              (strcmp (validation, "localxsv") == 0) ||
              (strcmp (validation, "onlinexsv") == 0) ))
            protocolVersion = getProtocolVersion (verb, datafilename, archive);
            
         // force xsv if protocol version is 1.0
         if (strcmp (protocolVersion, "1.0") == 0)
         {
            if (strcmp (validation, "localxerces") == 0)
               validation = "localxsv";
            else if (strcmp (validation, "onlinexerces") == 0)
               validation = "onlinexsv";
         }
      
         // perform schema validation
         char *xsverror;         
         if (strcmp (validation, "localxerces") == 0)
            xsverror = ValidateXerces (protocolVersion, datafilename, xsvfilename, verb, metadataPrefix, 1);
         else if (strcmp (validation, "onlinexerces") == 0)
            xsverror = ValidateXerces (protocolVersion, datafilename, xsvfilename, verb, metadataPrefix, 0);
         else if (strcmp (validation, "localxsv") == 0)
            xsverror = ValidateXSV (protocolVersion, datafilename, xsvfilename, verb, metadataPrefix, 1);
         else if (strcmp (validation, "onlinexsv") == 0)
            xsverror = ValidateXSV (protocolVersion, datafilename, xsvfilename, verb, metadataPrefix, 0);
         else
            xsverror = NULL;

         // make frame for dual display
         if (strcmp (display, "both") == 0)
         {
            cout << "<table width=\"100%\" cellspacing=2 cellpadding=10 border=2>"
                 << "<tr bgcolor=\"#00cc00\"><th align=left>"
                 << Translate ("Parsed Output") << "</th><th align=left>" 
                 << Translate ("Raw XML Output") << "</th></tr>"
                 << "<tr><td valign=top>";
         }

         // report schema errors
         if (xsverror)
         {
            cout << "<h3>"
                 << Translate ("XML Schema Validation Error !")
                 << "</h3><p>";
            cout << xsverror << "</p>";
            DumpFile (xsvfilename);
         }
         
         // if no schema errors ...
         else
         {
            // parse XML file
            if ((strcmp (display, "parsed") == 0) || (strcmp (display, "both") == 0))
            {
               if (runparser (&tl, datafilename))
               {
                  char *requestURL, *responseDate;
                  requestURL = NULL;
                  responseDate = NULL;
                  char buffer[2048];
                  
                  // run correct parser
                  if ((strcmp (protocolVersion, "1.0") == 0) ||
                      (strcmp (protocolVersion, "1.1") == 0))
                  {
                     if (strcmp (verb, "Identify") == 0)
                        ProcessIdentify  (&tl, &requestURL, &responseDate);
                     else if (strcmp (verb, "ListMetadataFormats") == 0)
                        ProcessListMetadataFormats  (&tl, identifier, &requestURL, &responseDate);
                     else if (strcmp (verb, "ListSets") == 0)
                        ProcessListSets  (&tl, &requestURL, &responseDate);            
                     else if (strcmp (verb, "ListIdentifiers") == 0)
                        ProcessListIdentifiers  (&tl, &requestURL, &responseDate);            
                     else if (strcmp (verb, "ListRecords") == 0)
                        ProcessListRecords  (&tl, &requestURL, &responseDate);            
                     else if (strcmp (verb, "GetRecord") == 0)
                        ProcessGetRecord  (&tl, &requestURL, &responseDate);
                  }
                  else if (strcmp (protocolVersion, "odl") == 0)
                  {
                     if (strcmp (verb, "Identify") == 0)
                        ProcessXIdentify  (&tl, &requestURL, &responseDate);
                     else if (strcmp (verb, "ListMetadataFormats") == 0)
                        ProcessXListMetadataFormats  (&tl, identifier, &requestURL, &responseDate);
                     else if (strcmp (verb, "ListSets") == 0)
                        ProcessXListSets  (&tl, &requestURL, &responseDate);            
                     else if (strcmp (verb, "ListIdentifiers") == 0)
                        ProcessXListIdentifiers  (&tl, &requestURL, &responseDate);            
                     else if (strcmp (verb, "ListRecords") == 0)
                        ProcessXListRecords  (&tl, &requestURL, &responseDate);            
                     else if (strcmp (verb, "GetRecord") == 0)
                        ProcessXGetRecord  (&tl, &requestURL, &responseDate);
                     else if (strcmp (verb, "PutRecord") == 0)
                        ProcessXPutRecord (&tl, &requestURL, &responseDate);
                  }
                  else
                  {
                     ProcessOAIPMH (&tl, &requestURL, &responseDate, verb, identifier, buffer);
                  }
                     
                  // check for existence of requestURL and responseDate
                  char RUError[1024];
                  strcpy (RUError, "requestURL ");
                  strcat (RUError, Translate ("missing !"));
                  char RDError[1024];
                  strcpy (RDError, "responseDate ");
                  strcat (RDError, Translate ("missing !"));
                  if (requestURL == NULL)
                     requestURL = RUError;
                  if (responseDate == NULL)
                     responseDate = RDError;
                  cout << "<hr><b>" << Translate ("Request") 
                       << " : </b>" << requestURL;
                  cout << "<br><b>" << Translate ("Response Date") 
                       << " : </b>" << responseDate;
               }
            }
            
            // or just display it plain
            else if (strcmp (display, "raw") == 0)
            {
               DumpFile (datafilename);
            }
         }
         
         // close frame if displaying in dual mode
         if (strcmp (display, "both") == 0)
         {
            cout << "</td><td valign=top>";
            DumpFile (datafilename);
            cout << "</td></tr></table>";
         }
         
         unlink (xsvfilename);
      }
      
      unlink (datafilename);
      unlink (errorfilename);
   }
   
   cout << "<hr>";
   
   // output bottom of screen forms/links
   OutputMenu (language, headercolor, blockcolor, display, validation, 
               from, until, metadataPrefix, identifier, set, resumptionToken,
               metadata, sets, protocolVersion);

   OutputFooters (language, bgcolor, headercolor, blockcolor);
   
   cout.flush ();
   return 0;
}

	